@@ -40,6 +40,8 @@ import sys
 import json
 import re
 
+from requests.models import Response
+from typing import Dict
 from urllib.parse import quote_plus
 from io import StringIO
 
@@ -63,16 +65,6 @@ from canalyst_candas.configuration.config import Config
 from canalyst_candas.settings import CONFIG
 
 
-def get_ticker_list(config, ticker=""):
-    GET = Getter(config=config)
-    path_name = f"DATA/cwl20.csv"
-    df = GET.get_csv_from_s3(path_name)
-    df = df.drop(columns=["CSIN"])
-    if ticker != "":
-        df = df.loc[df["Bloomberg Ticker"] == ticker]
-    return df
-
-
 def help():
     print("Canalyst Candas help")
     print(
@@ -117,6 +109,44 @@ class Search:
         self.config = config
         self.df_search = None
         self.df_guidance = None
+        self.ticker_list = None
+
+    def remove_stopwords(self, sentence):
+        my_stopwords = [
+            "in",
+            "Change",
+            "YoY",
+            ",",
+            "%",
+            "(Calculated)",
+            "Total",
+            "growth",
+            "Growth",
+            "Average",
+            "(QoQ)",
+            "QoQ",
+            "mm",
+        ]
+        tokens = sentence.split(" ")
+        tokens_filtered = [word for word in tokens if not word in my_stopwords]
+        tokens_filtered = [
+            "".join(c for c in s if c not in string.punctuation)
+            for s in tokens_filtered
+        ]
+        return (" ").join(tokens_filtered)
+
+    def get_ticker_list(self, ticker=""):
+        if self.ticker_list is None:
+            GET = Getter(config=self.config)
+            path_name = f"DATA/cwl20.csv"
+            df = GET.get_csv_from_s3(path_name)
+            df = df.drop(columns=["CSIN"])
+            self.ticker_list = df
+        else:
+            df = self.ticker_list
+        if ticker != "":
+            df = df.loc[df["Bloomberg Ticker"] == ticker]
+        return df
 
     def help(self):
         print("search_time_series syntax:")
@@ -232,11 +262,14 @@ class Search:
                 )
             ]
         if time_series_description != "":
-            df = df.loc[
-                df["time_series_description"].str.contains(
-                    time_series_description, case=False, regex=True
-                )
-            ]
+
+            ts_lower = time_series_description.lower()
+            split_strings = ts_lower.split(" ")
+
+            df["ts_lower"] = df["time_series_description"].str.lower()
+            df["search_matches"] = df["ts_lower"].str.count("|".join(split_strings))
+            df = df.loc[df["search_matches"] > 0]
+
         if is_driver != "":
             df = df.loc[df["is_driver"] == is_driver]
         if unit_type != "":
@@ -261,8 +294,11 @@ class Search:
                 "fiscal_or_annual",
                 "unit_type",
                 "is_driver",
+                "search_matches",
             ]
-        ].sort_values(["ticker", "time_series_description"])
+        ].sort_values(
+            ["search_matches", "ticker", "time_series_description"], ascending=False
+        )
 
 
 # WIP Class for parsing json revenue builds in graph structure (as opposed to tree)
@@ -873,8 +909,7 @@ class ModelSet:
             return
 
         if ticker in self.ticker_list:
-            self.models[df].create_time_series_chart(time_series_name)
-
+            self.models[ticker].create_time_series_chart(time_series_name)
         else:
             print("Please choose a ticker in this ModelSet's ticker list")
         return
@@ -885,11 +920,20 @@ class ModelSet:
             for t in ticker:
                 if ticker in self.ticker_list:
                     df = self.models[ticker].guidance()
+                    df["ticker"] = ticker
+                    col_name = "ticker"
+                    first_col = df.pop(col_name)
+                    df.insert(0, col_name, first_col)
                     list_df.append(df)
             return pd.concat(list_df)
         else:
             if ticker in self.ticker_list:
-                return self.models[ticker].guidance()
+                df = self.models[ticker].guidance()
+                df["ticker"] = ticker
+                col_name = "ticker"
+                first_col = df.pop(col_name)
+                df.insert(0, col_name, first_col)
+                return df
             else:
                 print("Please choose a ticker in this ModelSet's ticker list")
         return
@@ -1140,6 +1184,7 @@ class ModelSet:
 
         return df
 
+    # MODELSET fit()
     def fit(self, params, return_series):
         # ticker period time_series_name value new_value
         dict_summary = {}
@@ -1187,13 +1232,19 @@ class ModelSet:
             for ticker, data in dict_data.items()
         )
 
+        for resp in res:
+            self.models[ticker].record_scenario_url(resp)
+
         ticker_list = list(set(list(df["ticker"])))
 
         for ticker in ticker_list:
             # print(return_series)
-            self.models[ticker].model_fit(return_series)
-            # try:
-            dict_summary[ticker] = self.models[ticker].summary()
+            try:
+                self.models[ticker].model_fit(return_series)
+                # try:
+                dict_summary[ticker] = self.models[ticker].summary()
+            except:
+                print("Error on fit for " + return_series)
             # except:
             #    self.log.write("Missing scenario for " + ticker)
 
@@ -1548,6 +1599,11 @@ class ModelMap:
     def create_time_series_chart(self, time_series_name):
         # get needed data from dataframe
         df = self.df[self.df["time_series_name"] == time_series_name]
+
+        if df.shape[0] == 0:
+            print("Time series not found")
+            return None
+
         df = df[df["period_name"].str.contains("Q")].sort_values("period_end_date")
         # use subset=['value'] to only drop rows with a null value
         df = df.dropna(subset=["value"])
@@ -1595,6 +1651,7 @@ class Model:
         self.log = LogFile(default_dir=self.config.default_dir)
 
         self.set_new_uuid()
+        self.scenario_url_map: Dict[str, str] = {}
 
         if self.force == True:
             print("model_frame")
@@ -1796,11 +1853,15 @@ class Model:
 
         return df
 
-    def create_time_series_chart(self, time_series_name):
+    def create_time_series_chart(self, time_series_name):  # MODEL
         # get needed data from dataframe
         df = self._model_frame[
             self._model_frame["time_series_name"] == time_series_name
         ]
+        if df.shape[0] == 0:
+            print("Time series not found")
+            return None
+
         df = df[df["period_name"].str.contains("Q")].sort_values("period_end_date")
         # use subset=['value'] to only drop rows with a null value
         df = df.dropna(subset=["value"])
@@ -1818,7 +1879,8 @@ class Model:
             labels={"period_name": xlabel, "value": ylabel},
         )
         fig.add_vline(x=self.mrq()["period_name"][0])
-        return fig
+        fig.show()
+        return
 
     def create_model_map(
         self,
@@ -1882,6 +1944,8 @@ class Model:
             self.config.mds_host,
         )
 
+        self.record_scenario_url(response)
+
         return
 
     def show_model_map(
@@ -1894,7 +1958,7 @@ class Model:
         mm = ModelMap(
             config=self.config,
             model=self,
-            time_series=time_series,
+            time_series_name=time_series,
             tree=tree,
             notebook=notebook,
             common_time_series=self._common_time_series_names,
@@ -2385,17 +2449,16 @@ class Model:
             f"{SCENARIO_URL.format(csin=self.csin, version=self.latest_version)}"
         ) + "?page_size=200"
 
-        scenario_response = get_request(scenario_url, headers=self.api_headers)
-
-        scenario_json = scenario_response.json()
-
-        scenario_id_url = map_scenario_urls(scenario_json).get(self.uuid)
+        if self.scenario_url_map.get(self.uuid) is None:
+            scenario_response = get_request(scenario_url, headers=self.api_headers)
+            scenario_json = scenario_response.json()
+            scenario_id_url = map_scenario_urls(scenario_json).get(self.uuid)
+        else:
+            scenario_id_url = self.scenario_url_map[self.uuid]
 
         if scenario_id_url is None:
 
-            print(
-                "Scenario ID for " + scenario_url + " is None.  Perhaps max scenarios."
-            )
+            print("Scenario ID for " + scenario_url + " is None.")
 
             return
 
@@ -2457,3 +2520,15 @@ class Model:
         self.scenario_df = pd.concat(all_df)
         print("Done")
         return
+
+    def record_scenario_url(self, scenario_response: Response) -> None:
+        if scenario_response is None:
+            return None
+        """
+        Extract scenario name and location from the response and add to scenario url dictionary
+        """
+        scenario_name = self.uuid
+        scenario_url = scenario_response.headers["Location"]
+
+        if self.scenario_url_map.get(scenario_name) is None:
+            self.scenario_url_map[scenario_name] = scenario_url
