@@ -19,6 +19,11 @@ import json
 
 import os
 import matplotlib.pyplot as plt
+
+# plt.style.use("dark_background")
+# plt.style.use("Solarize_Light2")
+plt.style.use("fivethirtyeight")
+
 from bs4 import BeautifulSoup
 import pandas as pd
 from datetime import datetime
@@ -503,6 +508,7 @@ def plot_prices_against_time_series(
 ):
     ticker = df_data.iloc[0]["ticker"]
     df_filter = df_data.loc[df_data["time_series_name"] == time_series_name]
+    time_series_description = df_data.iloc[0]["time_series_description"]
     df_plot = pd.merge(
         df_prices[["adjclose", "price_date"]],
         df_filter[["time_series_name", "earnings_date", value_name]],
@@ -520,11 +526,18 @@ def plot_prices_against_time_series(
     colors = {
         "red": "#ff207c",
         "grey": "#42535b",
-        "blue": "#207cff",
+        # "blue": "#FFD200",
+        "blue": "#00838F",
         "orange": "#ffa320",
         "green": "#00ec8b",
     }
-    plt.rc("figure", figsize=(10, 7))
+    plt.rc("figure", figsize=(12, 9))
+
+    # csfont = {'fontname':'Helvetica'}
+    # hfont = {'fontname':'Roboto Regular'}
+
+    # plt.title(str(ticker) + " price vs " + str(time_series_description),**csfont)
+    # plt.xlabel('xlabel', **hfont)
 
     config_ticks = {"size": 14, "color": colors["grey"], "labelcolor": colors["grey"]}
     config_title = {"size": 18, "color": colors["grey"], "ha": "left", "va": "baseline"}
@@ -533,6 +546,7 @@ def plot_prices_against_time_series(
 
     fig, axes = plt.subplots(2, 1, gridspec_kw={"height_ratios": [3, 1]}, sharex=True)
     fig.tight_layout(pad=3)
+    fig.suptitle(str(ticker) + " price vs " + str(time_series_description), fontsize=16)
 
     date = df_plot["price_date"]
     close = df_plot["adjclose"]
@@ -552,7 +566,7 @@ def plot_prices_against_time_series(
     )
 
     plot_vol = axes[1]
-    plot_vol.bar(date, vol, width=15, color="darkgrey")
+    plot_vol.bar(date, vol, width=25, color="darkgrey")
     plot_vol.set_ylabel(time_series_name, fontsize=14)
     plot_vol.yaxis.set_label_position("left")
     plot_vol.yaxis.label.set_color(colors["grey"])
@@ -562,7 +576,7 @@ def plot_prices_against_time_series(
     plot_price.set_ylabel("Price (in USD)", fontsize=14)
     plot_price.yaxis.set_label_position("right")
     plot_price.yaxis.label.set_color(colors["grey"])
-    plot_price.grid(axis="y", color="gainsboro", linestyle="-", linewidth=0.5)
+    plot_price.grid(axis="y", color="gainsboro", linestyle="-", linewidth=1)
     plot_price.set_axisbelow(True)
 
     format_borders(plot_price, colors)
@@ -593,11 +607,11 @@ def plot_eps_surprise_data(df, alpha_days="alpha_10_days"):
     colors = {
         "red": "#ff207c",
         "grey": "#1d2224",
-        "blue": "#207cff",
+        "blue": "#00838F",
         "orange": "#ffa320",
         "green": "#00ec8b",
     }
-    plt.rc("figure", figsize=(10, 7))
+    plt.rc("figure", figsize=(12, 9))
 
     config_ticks = {"size": 14, "color": colors["grey"], "labelcolor": colors["grey"]}
     config_title = {"size": 18, "color": colors["grey"], "ha": "left", "va": "baseline"}
@@ -619,7 +633,10 @@ def plot_eps_surprise_data(df, alpha_days="alpha_10_days"):
     plt.gca().yaxis.set_label_position("right")
     plt.grid(axis="y", color="gainsboro", linestyle="-", linewidth=0.5)
     plt.legend(loc="upper left")
-    plt.title(ticker + " earnings suprise and alpha correlation:" + str(i_correlation))
+    plt.title(
+        ticker + " earnings suprise and alpha correlation:" + str(i_correlation),
+        fontsize=16,
+    )
     format_borders(plt.gca(), colors)
 
     return
@@ -675,6 +692,7 @@ def regress_dataframe_time_series_groups(
 
     df = df.loc[df["rsquared"] < 0.98]
     df = df.loc[df["rsquared"] > 0.01]
+    df = df.loc[df["n_periods"] >= n_periods]
     return df
 
 
@@ -777,7 +795,7 @@ def rolling_r_squared(
     df = df_data[
         df_data["time_series_name"] == time_series_name
     ]  # [['period_name','value','earnings_date','alpha_10_day']]
-
+    time_series_description = df_data.iloc[0]["time_series_description"]
     model = RollingOLS.from_formula(
         "value ~ " + str(y_name),
         data=df[["value", y_name]],
@@ -807,27 +825,26 @@ def rolling_r_squared(
     if plot == False:
         return df
 
-    import matplotlib.pyplot as plt
-
     colors = {
         "red": "#ff207c",
-        "grey": "#42535b",
-        "blue": "#207cff",
+        "grey": "#C3C2C3",
+        "blue": "#00838F",
         "orange": "#ffa320",
         "green": "#00ec8b",
     }
-    plt.rc("figure", figsize=(10, 7))
+    plt.rc("figure", figsize=(12, 9))
 
     fig, (ax1, ax2, ax3) = plt.subplots(nrows=3, sharex=True)
-    fig.tight_layout(pad=3)
+    fig.suptitle(time_series_description, fontsize=16)
+    fig.tight_layout(pad=1)
 
-    ax1.plot(df["slope"], label="Regression Slope", color=colors["blue"])
+    ax1.plot(df["slope"], label="Regression Slope", color=colors["blue"], linewidth=2)
     ax1.legend(loc="upper left")
 
-    ax2.plot(df["rsquared"], label="R Squared", color=colors["blue"])
+    ax2.plot(df["rsquared"], label="R Squared", color=colors["blue"], linewidth=2)
     ax2.legend(loc="upper left")
 
-    ax3.plot(df["value"], label="Time Series Value", color=colors["blue"])
+    ax3.plot(df["value"], label="Time Series Value", color=colors["blue"], linewidth=2)
     ax3.legend(loc="upper left")
 
     import warnings
@@ -838,7 +855,7 @@ def rolling_r_squared(
 
     ax1.get_shared_x_axes().join(ax1, ax2, ax3)
 
-    ax1.title.set_text(time_series_name)
+    # ax1.title.set_text(time_series_description,fontsize=14)
     format_borders(ax1, colors)
     format_borders(ax2, colors)
     format_borders(ax3, colors)
@@ -846,11 +863,13 @@ def rolling_r_squared(
     plt.show()
 
 
-def remove_stopwords(sentence):
-    my_stopwords = ["in", "Change", "YoY", ",", "%", "(Calculated)", "Total", "growth"]
-    tokens = sentence.split(" ")
-    tokens_filtered = [word for word in tokens if not word in my_stopwords]
-    tokens_filtered = [
-        "".join(c for c in s if c not in string.punctuation) for s in tokens_filtered
-    ]
-    return (" ").join(tokens_filtered)
+def get_earnings_estimates(ticker):
+    return si.get_analysts_info(ticker)["Earnings Estimate"]
+
+
+def get_revenue_estimates(ticker):
+    return si.get_analysts_info(ticker)["Revenue Estimate"]
+
+
+def get_eps_revisions(ticker):
+    return si.get_analysts_info(ticker)["EPS Revisions"]
