@@ -2,7 +2,7 @@
 Connection module
 """
 from typing import Any
-from canalyst_candas.candas import Model, ModelMap, ModelSet
+from canalyst_candas.candas import Model, ModelMap, ModelSet, Search
 from canalyst_candas.configuration.config import Config, resolve_config
 
 
